@@ -9,6 +9,7 @@
            , UndecidableInstances
            , ScopedTypeVariables
            , InstanceSigs
+           , Rank2Types
  #-}
 -- | This is an internal module, anything exported by this module
 -- may change without a major version bump.  Please use only
@@ -50,17 +51,16 @@ module Database.Esqueleto.Internal.Sql
   , veryUnsafeCoerceSqlExprValueList
   ) where
 
-import Control.Applicative (Applicative(..), (<$>), (<$))
 import Control.Arrow ((***), first)
 import Control.Exception (throw, throwIO)
-import Control.Monad (ap, MonadPlus(..), liftM)
+import Control.Monad (ap, MonadPlus(..), join, void)
 import Control.Monad.IO.Class (MonadIO(..))
 import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.Resource (MonadResource)
 import Data.Acquire (with, allocateAcquire, Acquire)
 import Data.Int (Int64)
 import Data.List (intersperse)
-import Data.Monoid (Last(..), Monoid(..), (<>))
+import Data.Monoid (Last(..), (<>))
 import Data.Proxy (Proxy(..))
 import Database.Esqueleto.Internal.PersistentImport
 import Database.Persist.Sql.Util (entityColumnNames, entityColumnCount, parseEntityValues, isIdField, hasCompositeKey)
@@ -76,7 +76,6 @@ import qualified Data.Text.Lazy.Builder as TLB
 
 import Database.Esqueleto.Internal.Language
 
-
 -- | SQL backend for @esqueleto@ using 'SqlPersistT'.
 newtype SqlQuery a =
   Q { unQ :: W.WriterT SideData (S.State IdentState) a }
@@ -647,7 +646,7 @@ unsafeSqlBinOpComposite op sep a b = ERaw Parens $ compose (listify a) (listify
 
     deconstruct :: (TLB.Builder, [PersistValue]) -> ([TLB.Builder], [PersistValue])
     deconstruct ("?", [PersistList vals]) = (replicate (length vals) "?", vals)
-    deconstruct (b, []) = (TLB.fromLazyText <$> TL.splitOn "," (TLB.toLazyText b), [])
+    deconstruct (b', []) = (TLB.fromLazyText <$> TL.splitOn "," (TLB.toLazyText b'), [])
     deconstruct x = err $ "cannot deconstruct " ++ show x ++ "."
 
     compose f1 f2 info
@@ -666,7 +665,7 @@ unsafeSqlBinOpComposite op sep a b = ERaw Parens $ compose (listify a) (listify
 -- | (Internal) A raw SQL value.  The same warning from
 -- 'unsafeSqlBinOp' applies to this function as well.
 unsafeSqlValue :: TLB.Builder -> SqlExpr (Value a)
-unsafeSqlValue v = ERaw Never $ \_ -> (v, mempty)
+unsafeSqlValue v = ERaw Never $ const (v, mempty)
 {-# INLINE unsafeSqlValue #-}
 
 
@@ -754,10 +753,10 @@ rawSelectSource :: ( SqlSelect a r
                    , MonadIO m2 )
                  => Mode
                  -> SqlQuery a
-                 -> SqlPersistT m1 (Acquire (C.Source m2 r))
+                 -> SqlReadT m1 (Acquire (C.Source m2 r))
 rawSelectSource mode query =
       do
-        conn <- R.ask
+        conn <- persistBackend <$> R.ask
         res <- run conn
         return $ (C.$= massage) `fmap` res
     where
@@ -783,12 +782,9 @@ selectSource :: ( SqlSelect a r
                 , MonadResource m )
              => SqlQuery a
              -> C.Source (SqlPersistT m) r
-selectSource query = do
-    src <- lift $ do
-        res <- rawSelectSource SELECT query
-        fmap snd $ allocateAcquire res
-    src
-
+selectSource query = join . lift $ do
+  res <- rawSelectSource SELECT query
+  snd <$> allocateAcquire res
 
 -- | Execute an @esqueleto@ @SELECT@ query inside @persistent@'s
 -- 'SqlPersistT' monad and return a list of rows.
@@ -833,7 +829,7 @@ selectSource query = do
 -- @SqlExpr (Entity Person)@.
 select :: ( SqlSelect a r
           , MonadIO m )
-       => SqlQuery a -> SqlPersistT m [r]
+       => SqlQuery a -> SqlReadT m [r]
 select query = do
     res <- rawSelectSource SELECT query
     conn <- R.ask
@@ -863,8 +859,8 @@ selectDistinct = select . distinct
 
 -- | (Internal) Run a 'C.Source' of rows.
 runSource :: Monad m =>
-             C.Source (SqlPersistT m) r
-          -> SqlPersistT m [r]
+             C.Source (R.ReaderT backend m) r
+          -> R.ReaderT backend m [r]
 runSource src = src C.$$ CL.consume
 
 
@@ -873,12 +869,12 @@ runSource src = src C.$$ CL.consume
 
 -- | (Internal) Execute an @esqueleto@ statement inside
 -- @persistent@'s 'SqlPersistT' monad.
-rawEsqueleto :: ( MonadIO m, SqlSelect a r )
+rawEsqueleto :: ( MonadIO m, SqlSelect a r, IsSqlBackend backend)
            => Mode
            -> SqlQuery a
-           -> SqlPersistT m Int64
+           -> R.ReaderT backend m Int64
 rawEsqueleto mode query = do
-  conn <- R.ask
+  conn <- persistBackend <$> R.ask
   uncurry rawExecuteCount $
     first builderToText $
     toRawSql mode (conn, initialIdentState) query
@@ -908,14 +904,13 @@ rawEsqueleto mode query = do
 -- @
 delete :: ( MonadIO m )
        => SqlQuery ()
-       -> SqlPersistT m ()
-delete = liftM (const ()) . deleteCount
-
+       -> SqlWriteT m ()
+delete = void . deleteCount
 
 -- | Same as 'delete', but returns the number of rows affected.
 deleteCount :: ( MonadIO m )
             => SqlQuery ()
-            -> SqlPersistT m Int64
+            -> SqlWriteT m Int64
 deleteCount = rawEsqueleto DELETE
 
 
@@ -927,22 +922,21 @@ deleteCount = rawEsqueleto DELETE
 -- Example of usage:
 --
 -- @
--- 'update' $ \p -> do
+-- 'update' $ \\p -> do
 -- 'set' p [ PersonAge '=.' 'just' ('val' thisYear) -. p '^.' PersonBorn ]
 -- 'where_' $ isNothing (p '^.' PersonAge)
 -- @
 update :: ( MonadIO m
           , SqlEntity val )
        => (SqlExpr (Entity val) -> SqlQuery ())
-       -> SqlPersistT m ()
-update = liftM (const ()) . updateCount
-
+       -> SqlWriteT m ()
+update = void . updateCount
 
 -- | Same as 'update', but returns the number of rows affected.
 updateCount :: ( MonadIO m
                , SqlEntity val )
             => (SqlExpr (Entity val) -> SqlQuery ())
-            -> SqlPersistT m Int64
+            -> SqlWriteT m Int64
 updateCount = rawEsqueleto UPDATE . from
 
 
@@ -961,7 +955,9 @@ builderToText = TL.toStrict . TLB.toLazyTextWith defaultChunkSize
 -- @esqueleto@, instead of manually using this function (which is
 -- possible but tedious), you may just turn on query logging of
 -- @persistent@.
-toRawSql :: SqlSelect a r => Mode -> IdentInfo -> SqlQuery a -> (TLB.Builder, [PersistValue])
+toRawSql
+  :: (IsSqlBackend backend, SqlSelect a r)
+  => Mode -> (backend, IdentState) -> SqlQuery a -> (TLB.Builder, [PersistValue])
 toRawSql mode (conn, firstIdentState) query =
   let ((ret, sd), finalIdentState) =
         flip S.runState firstIdentState $
@@ -980,7 +976,7 @@ toRawSql mode (conn, firstIdentState) query =
       -- that were used) to the subsequent calls.  This ensures
       -- that no name clashes will occur on subqueries that may
       -- appear on the expressions below.
-      info = (conn, finalIdentState)
+      info = (persistBackend conn, finalIdentState)
   in mconcat
       [ makeInsertInto info mode ret
       , makeSelect     info mode distinctClause ret
@@ -1034,7 +1030,7 @@ makeSelect info mode_ distinctClause ret = process mode_
         DistinctOn exprs -> first (("SELECT DISTINCT ON (" <>) . (<> ") ")) $
                             uncommas' (processExpr <$> exprs)
       where processExpr (EDistinctOn f) = materializeExpr info f
-    withCols v = v <> (sqlSelectCols info ret)
+    withCols v = v <> sqlSelectCols info ret
     plain    v = (v, [])
 
 
@@ -1102,13 +1098,19 @@ makeGroupBy :: IdentInfo -> GroupByClause -> (TLB.Builder, [PersistValue])
 makeGroupBy _ (GroupBy []) = (mempty, [])
 makeGroupBy info (GroupBy fields) = first ("\nGROUP BY " <>) build
   where
-    build = uncommas' $ map (\(SomeValue (ERaw _ f)) -> f info) fields
+    build :: (TLB.Builder, [PersistValue])
+    build = uncommas' $ map match fields
+
+    match :: SomeValue SqlExpr -> (TLB.Builder, [PersistValue])
+    match (SomeValue (ERaw _ f)) = f info
+    match (SomeValue (ECompositeKey f)) = (mconcat $ f info, mempty)
 
 makeHaving :: IdentInfo -> WhereClause -> (TLB.Builder, [PersistValue])
 makeHaving _    NoWhere                    = mempty
 makeHaving info (Where (ERaw _ f))         = first ("\nHAVING " <>) (f info)
-makeHaving _    (Where (ECompositeKey _ )) = unexpectedCompositeKeyError "makeHaving"
+makeHaving _    (Where (ECompositeKey _)) = unexpectedCompositeKeyError "makeHaving"
 
+-- makeHaving, makeWhere and makeOrderBy
 makeOrderBy :: IdentInfo -> [OrderByClause] -> (TLB.Builder, [PersistValue])
 makeOrderBy _    [] = mempty
 makeOrderBy info os = first ("\nORDER BY " <>) . uncommas' $ concatMap mk os
@@ -1119,7 +1121,7 @@ makeOrderBy info os = first ("\nORDER BY " <>) . uncommas' $ concatMap mk os
       let fs = f info
           vals = repeat []
       in zip (map (<> orderByType t) fs) vals
-    mk EOrderRandom = [first ((<> "RANDOM()")) mempty]
+    mk EOrderRandom = [first (<> "RANDOM()") mempty]
     orderByType ASC  = " ASC"
     orderByType DESC = " DESC"
 
@@ -1212,8 +1214,7 @@ instance PersistEntity a => SqlSelect (SqlExpr (Entity a)) (Entity a) where
               in (process ed, mempty)
   sqlSelectColCount = entityColumnCount . entityDef . getEntityVal
   sqlSelectProcessRow = parseEntityValues ed
-    where ed = entityDef $ getEntityVal $ (Proxy :: Proxy (SqlExpr (Entity a)))
-
+    where ed = entityDef $ getEntityVal (Proxy :: Proxy (SqlExpr (Entity a)))
 
 getEntityVal :: Proxy (SqlExpr (Entity a)) -> Proxy a
 getEntityVal = const Proxy
@@ -1745,17 +1746,17 @@ to16 ((a,b),(c,d),(e,f),(g,h),(i,j),(k,l),(m,n),(o,p)) = (a,b,c,d,e,f,g,h,i,j,k,
 --
 -- /Since: 2.4.2/
 insertSelect :: (MonadIO m, PersistEntity a) =>
-  SqlQuery (SqlExpr (Insertion a)) -> SqlPersistT m ()
-insertSelect = liftM (const ()) . insertSelectCount
+  SqlQuery (SqlExpr (Insertion a)) -> SqlWriteT m ()
+insertSelect = void . insertSelectCount
 
 -- | Insert a 'PersistField' for every selected value, return the count afterward
 insertSelectCount :: (MonadIO m, PersistEntity a) =>
-  SqlQuery (SqlExpr (Insertion a)) -> SqlPersistT m Int64
+  SqlQuery (SqlExpr (Insertion a)) -> SqlWriteT m Int64
 insertSelectCount = rawEsqueleto INSERT_INTO . fmap EInsertFinal
 
 
 -- | Insert a 'PersistField' for every unique selected value.
 insertSelectDistinct :: (MonadIO m, PersistEntity a) =>
-  SqlQuery (SqlExpr (Insertion a)) -> SqlPersistT m ()
+  SqlQuery (SqlExpr (Insertion a)) -> SqlWriteT m ()
 insertSelectDistinct = insertSelect . distinct
 {-# DEPRECATED insertSelectDistinct "Since 2.2.4: use 'insertSelect' and 'distinct'." #-}
