@@ -18,21 +18,19 @@
  #-}
 module Main (main) where
 
-import Control.Applicative ((<$>))
-import Control.Arrow ((&&&))
-import Control.Exception (IOException)
 import Control.Monad (forM_, replicateM, replicateM_, void)
 import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad.Logger (MonadLogger(..), runStderrLoggingT, runNoLoggingT)
 import Control.Monad.Trans.Control (MonadBaseControl(..))
 import Control.Monad.Trans.Reader (ReaderT)
 import Data.Char (toLower, toUpper)
-import Data.List (sortBy)
 import Data.Monoid ((<>))
-import Data.Ord (comparing)
 import Database.Esqueleto
 #if   defined (WITH_POSTGRESQL)
 import Database.Persist.Postgresql (withPostgresqlConn)
+import Data.Ord (comparing)
+import Control.Arrow ((&&&))
+import qualified Database.Esqueleto.PostgreSQL as EP
 #elif defined (WITH_MYSQL)
 import Database.Persist.MySQL ( withMySQLConn
                               , connectHost
@@ -40,12 +38,9 @@ import Database.Persist.MySQL ( withMySQLConn
                               , connectUser
                               , connectPassword
                               , defaultConnectInfo)
-#else
+#endif
 import Database.Persist.Sqlite (withSqliteConn)
-#if MIN_VERSION_persistent_sqlite(2,1,3)
 import Database.Sqlite (SqliteException)
-#endif
-#endif
 import Database.Persist.TH
 import Test.Hspec
 
@@ -53,7 +48,6 @@ import qualified Control.Monad.Trans.Resource as R
 import qualified Data.List as L
 import qualified Data.Set as S
 import qualified Data.Text.Lazy.Builder as TLB
-import qualified Database.Esqueleto.PostgreSQL as EP
 import qualified Database.Esqueleto.Internal.Sql as EI
 
 
@@ -75,6 +69,19 @@ share [mkPersist sqlSettings, mkMigrate "migrateAll"] [persistUpperCase|
     title String
     authorId PersonId
     deriving Eq Show
+
+  Lord
+    county String
+    dogs Int Maybe
+    Primary county
+    deriving Show
+
+  Deed
+    contract String
+    ownerId LordId
+    Primary contract
+    deriving Show
+
   Follow
     follower PersonId
     followed PersonId
@@ -135,6 +142,10 @@ main = do
       p3 = Person "Mike"   (Just 17) Nothing   3
       p4 = Person "Livia"  (Just 17) (Just 18) 4
       p5 = Person "Mitch"  Nothing   Nothing   5
+      l1 = Lord "Cornwall" (Just 36)
+      l2 = Lord "Dorset" Nothing
+      l3 = Lord "Chester" (Just 17)
+
   hspec $ do
     describe "select" $ do
       it "works for a single value" $
@@ -154,7 +165,7 @@ main = do
 
       it "works for a single NULL value" $
         run $ do
-          ret <- select $ return $ nothing
+          ret <- select $ return nothing
           liftIO $ ret `shouldBe` [ Value (Nothing :: Maybe Int) ]
 
     describe "select/from" $ do
@@ -289,7 +300,7 @@ main = do
               number = 101
               Right thePk = keyFromValues [toPersistValue number]
           fcPk <- insert fc
-          [Entity _ ret] <- select $ from $ return
+          [Entity _ ret] <- select $ from return
           liftIO $ do
             ret `shouldBe` fc
             fcPk `shouldBe` thePk
@@ -831,7 +842,7 @@ main = do
                  distinctOn [don (bp ^. BlogPostAuthorId)] $ do
                  orderBy [asc (bp ^. BlogPostAuthorId), desc (bp ^. BlogPostTitle)]
                  return bp
-          liftIO $ ret `shouldBe` sortBy (comparing (blogPostAuthorId . entityVal)) [bpB, bpC]
+          liftIO $ ret `shouldBe` L.sortBy (comparing (blogPostAuthorId . entityVal)) [bpB, bpC]
 
       let slightlyLessSimpleTest q =
             run $ do
@@ -844,7 +855,7 @@ main = do
                      from $ \bp ->
                      q bp $ return bp
               let cmp = (blogPostAuthorId &&& blogPostTitle) . entityVal
-              liftIO $ ret `shouldBe` sortBy (comparing cmp) [bpA, bpB, bpC]
+              liftIO $ ret `shouldBe` L.sortBy (comparing cmp) [bpA, bpB, bpC]
       it "works on a slightly less simple example (two distinctOn calls, orderBy)" $
         slightlyLessSimpleTest $ \bp act ->
           distinctOn [don (bp ^. BlogPostAuthorId)] $
@@ -921,12 +932,8 @@ main = do
       it "throws an exception on SQLite with <2 arguments" $
         run (select $
              from $ \p -> do
-             return (coalesce [p ^. PersonAge]) :: SqlQuery (SqlExpr (Value (Maybe Int)))
-#if MIN_VERSION_persistent_sqlite(2,1,3)
-        ) `shouldThrow` (\(_ :: SqliteException) -> True)
-#else
-        ) `shouldThrow` (\(_ :: IOException) -> True)
-#endif
+             return (coalesce [p ^. PersonAge]) :: SqlQuery (SqlExpr (Value (Maybe Int))))
+        `shouldThrow` (\(_ :: SqliteException) -> True)
 #endif
 
     describe "text functions" $ do
@@ -1079,6 +1086,23 @@ main = do
                                   , (Entity p1k p1, Value 3)
                                   , (Entity p3k p3, Value 7) ]
 
+      it "GROUP BY works with COUNT and InnerJoin" $
+        run $ do
+          l1k <- insert l1
+          l2k <- insert l2
+          l3k <- insert l3
+          mapM_ (\k -> insert $ Deed k l1k) (map show [1..3])
+
+          mapM_ (\k -> insert $ Deed k l3k) (map show [4..10])
+
+          (ret :: [(Value (Key Lord), Value Int)]) <- select $ from $
+            \ ( lord `InnerJoin` deed ) -> do
+            on $ lord ^. LordId ==. deed ^. DeedOwnerId
+            groupBy (lord ^. LordId)
+            return (lord ^. LordId, count $ deed ^. DeedId)
+          liftIO $ ret `shouldBe` [ (Value l3k, Value 7)
+                                  , (Value l1k, Value 3) ]
+
       it "GROUP BY works with HAVING" $
         run $ do
           p1k <- insert p1
@@ -1187,7 +1211,6 @@ main = do
                  return p
           liftIO $ ret `shouldBe` [ Entity p2k p2 ]
 
-
     describe "list fields" $ do
       -- <https://github.com/prowdsponsor/esqueleto/issues/100>
       it "can update list fields" $
@@ -1371,9 +1394,7 @@ main = do
           let people = [p1, p2, p3, p4, p5]
           mapM_ insert people
           [Value ret] <-
-            select $
-            from $ \p -> do
-            return (EP.arrayAgg (p ^. PersonName))
+            select . from $ \p -> return (EP.arrayAgg (p ^. PersonName))
           liftIO $ L.sort ret `shouldBe` L.sort (map personName people)
 
       it "stringAgg looks sane" $
@@ -1396,10 +1417,11 @@ main = do
 
 
 insert' :: ( Functor m
-           , PersistStore (PersistEntityBackend val)
+           , BaseBackend backend ~ PersistEntityBackend val
+           , PersistStore backend
            , MonadIO m
            , PersistEntity val )
-        => val -> ReaderT (PersistEntityBackend val) m (Entity val)
+        => val -> ReaderT backend m (Entity val)
 insert' v = flip Entity v <$> insert v
 
 
@@ -1415,10 +1437,15 @@ cleanDB
   :: (forall m. RunDbMonad m
   => SqlPersistT (R.ResourceT m) ())
 cleanDB = do
+  delete $ from $ \(_ :: SqlExpr (Entity Foo))  -> return ()
+  delete $ from $ \(_ :: SqlExpr (Entity Bar))  -> return ()
+
   delete $ from $ \(_ :: SqlExpr (Entity BlogPost))   -> return ()
   delete $ from $ \(_ :: SqlExpr (Entity Follow))     -> return ()
   delete $ from $ \(_ :: SqlExpr (Entity Person))     -> return ()
 
+  delete $ from $ \(_ :: SqlExpr (Entity CcList))  -> return ()
+
   delete $ from $ \(_ :: SqlExpr (Entity ArticleTag)) -> return ()
   delete $ from $ \(_ :: SqlExpr (Entity Article))    -> return ()
   delete $ from $ \(_ :: SqlExpr (Entity Article2))   -> return ()
